﻿import es
import random
import gamethread
import datetime
import popuplib
import usermsg
import re


info                            = {}
info["name"]            = "AdvertLib"
info["version"]         = "2.0.2"
info["author"]          = "Brainsucker"
info["description"]     = "Sends Adverts in specified time intervals to avoid Chatspam."



class BaseAdvert(object):
        def __init__(self, advert_name, advert_text):
                self.enabled = True
                self.description = '' # Not really required. Just a little feature.
                self.min_interval = datetime.timedelta(seconds=45)
                self.name = advert_name
                self.text = advert_text
                self.filters = set() # Allows other Scripts to register a filter for each Advert. This filter allows to disable adverts for single players.
                # self.lang = {}
                # self.default_lang = 'en'
                self.last_sent = None
                
                AdvertManager.adverts.add(self) # Automaticly register itself to the manager (who creates an advert and does not want to use it?!)
                
        def unregister()(self):
                AdvertManager.adverts.remove(self)
                
        def enable(self):
                self.enabled = True
                
        def disable(self):
                self.enabled = False
                
        def filterPlayers(self):
                ulist = es.getUseridList()
                
                filtered = []
                for p in ulist:
                        for f in self.filters:
                                if callable(f):
                                        try:
                                                result = f(p, self.name)
                                        except:
                                                pass
                                        else:
                                                if not result:
                                                        filtered.append(p)
                for fp in filtered:
                        ulist.remove(fp)
                                
                return ulist
        
                
class ColoredAdvert(BaseAdvert):
        color_regex = re.compile('\$color', re.I)
        def __init__(self, advert_name, advert_text):
                self.colors = []
                BaseAdvert.__init__(self, advert_name, advert_text)
                
        def _formatText(self, colored=True):
                temp_text = self.text
                if colored:
                        for c in self.colors:
                                temp_text = ColoredAdvert.color_regex.sub(c, temp_text, 1)
                return ColoredAdvert.color_regex.sub('', temp_text)
                
                
class PopupAdvert(BaseAdvert):
        def __init__(self, advert_name, advert_text):
                BaseAdvert.__init__(self, advert_name, advert_text)
                self.duration = 0
                
                
        def send(self, users=None):
                if users is None:
                        users = self.filterPlayers()
                for p in users:
                        popuplib.quicksend(self.duration, p, self.text)
        
class ChatAdvert(ColoredAdvert):
        def __init__(self, advert_name, advert_text):
                ColoredAdvert.__init__(self, advert_name, advert_text)
                
                
        def send(self, users=None):
                if users is None:
                        users = self.filterPlayers()
                for p in users:
                        es.tell(p, '#multi', '#default'+self._formatText(colored=True))
        
class CenterAdvert(BaseAdvert):
        def __init__(self, advert_name, advert_text):
                BaseAdvert.__init__(self, advert_name, advert_text)
                self.duration = 5
                
        def send(self, users=None):
                if users is None:
                        users = self.filterPlayers()
                for p in users:
                        for x in xrange(self.duration):
                                gamethread.delayed(x, es.centertell, (p, self.text))
                
                
class HudhintAdvert(BaseAdvert):
        def __init__(self, advert_name, advert_text):
                BaseAdvert.__init__(self, advert_name, advert_text)
                
        def send(self, users=None):
                if users is None:
                        users = self.filterPlayers()
                for p in users:
                        usermsg.hudhint(p, self.text)
        
class Saytext2Advert(ColoredAdvert):
        def __init__(self, advert_name, advert_text):
                ColoredAdvert.__init__(self, advert_name, advert_text)
                self.index = 1
                
                
        def send(self, users=None):
                if users is None:
                        users = self.filterPlayers()
                for p in users:
                        usermsg.saytext2(p, str(self.index), self._formatText(colored=True))

class _AM():
        def __init__(self):
                self.advert_interval = datetime.timedelta(seconds=60) # Defines the amount of time between every Advert.
                self.adverts = set()
                self.random_adverts = False
                self.__running = False
                self.preAdverts = set()
                self.nextAdvert = None
                self._lastAdvert = None
                self._lastAdvertTime = None
                self.__advertcounter = 0
                self.__loopname = 'advertlib_advertloop'
                
                        
        def _loopAdverts(self):
                self.__running = True
                gamethread.delayedname(1, self.__loopname, self._loopAdverts)
                if es.getplayercount() == 0: # No Players - No Adverts
                        return
                n = datetime.datetime.today()
                if not self._lastAdvertTime is None:
                        diff = n - self._lastAdvertTime
                        if diff < self.advert_interval:
                                return
                enabled_adverts = filter(lambda a: bool(a.enabled), self.adverts)
                available_adverts = filter(lambda b: ((n - b.last_sent) >= b.min_interval) if not b.last_sent is None else True , enabled_adverts)
                if len(available_adverts) > 0:
                        for pa in self.preAdverts:
                                if callable(pa):
                                        try:
                                                pa()
                                        except:
                                                pass

                        
                        if self.nextAdvert:
                                na = self.nextAdvert
                                self.nextAdvert = None
                                
                        elif self.random_adverts:
                                na = random.choice(available_adverts)
                                
                        else:
                                if self.__advertcounter >= len(available_adverts):
                                        self.__advertcounter = 0
                                na = available_adverts[self.__advertcounter]
                                self.__advertcounter += 1
                                        
                                        
                        self._lastAdvert = na
                        self._lastAdvertTime = n
                        self.sendAdvert(na)
                        
        def sendAdvert(self, advert_instance):
                n = datetime.datetime.today()
                advert_instance.last_sent = n
                advert_instance.send()

        def start(self):
                if not self.__running:
                        self._loopAdverts()
                
        def stop(self):
                if self.__running:
                        gamethread.cancelDelayed(self.__loopname)
                        self.__running = False
                        
        def isRunning(self):
                return self.__running
        

# Initiate one AdvertManager which has to be used by all other Scripts.
        
AdvertManager = _AM()
# Start sending adverts after 10 seconds.
gamethread.delayed(10, AdvertManager.start, ())